@@ -1,5 +1,5 @@
 /* Gobby - GTK-based collaborative text editor
- * Copyright (C) 2008-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2008-2015 Armin Burgmeier <armin@arbur.net>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -102,7 +102,7 @@ private:
 	}
 
 	BrowserCommands& m_commands;
-	std::auto_ptr<UserJoin> m_userjoin;
+	std::unique_ptr<UserJoin> m_userjoin;
 
 	InfBrowser* m_browser;
 	gulong m_notify_status_handler;
@@ -222,9 +222,10 @@ Gobby::BrowserCommands::BrowserInfo::set_pending_chat(InfSessionProxy* proxy)
 	// the folder manager so that we can correctly show the document when
 	// enough users are available. Otherwise if no party would join a user
 	// before other users show up, the chat would never be shown.
-	std::auto_ptr<UserJoin::ParameterProvider> provider(
+	std::unique_ptr<UserJoin::ParameterProvider> provider(
 		new ParameterProvider(m_commands.m_preferences));
-	m_userjoin.reset(new UserJoin(m_browser, NULL, proxy, provider));
+	m_userjoin.reset(new UserJoin(m_browser, NULL, proxy,
+	                              std::move(provider)));
 
 	check_pending_chat();
 }
@@ -341,6 +342,34 @@ Gobby::BrowserCommands::BrowserCommands(Browser& browser,
 	m_set_browser_handler = g_signal_connect(
 		browser.get_store(), "set-browser",
 		G_CALLBACK(on_set_browser_static), this);
+
+	// Add already existing browsers
+	GtkTreeIter iter;
+	GtkTreeModel* treemodel = GTK_TREE_MODEL(browser.get_store());
+	for(gboolean have_entry =
+		gtk_tree_model_get_iter_first(treemodel, &iter);
+	    have_entry == TRUE;
+	    have_entry = gtk_tree_model_iter_next(treemodel, &iter))
+	{
+		InfBrowser* browser;
+
+		gtk_tree_model_get(
+			treemodel, &iter,
+			INF_GTK_BROWSER_MODEL_COL_BROWSER, &browser,
+			-1);
+
+		InfBrowserStatus browser_status;
+		g_object_get(
+			G_OBJECT(browser), "status",
+			&browser_status, NULL);
+
+		m_browser_map[browser] =
+			new BrowserInfo(*this, browser);
+		if(browser_status == INF_BROWSER_OPEN)
+			if(!create_chat_document(browser))
+				subscribe_chat(browser);
+		g_object_unref(browser);
+	}
 }
 
 Gobby::BrowserCommands::~BrowserCommands()
@@ -443,7 +472,7 @@ void Gobby::BrowserCommands::subscribe_chat(InfBrowser* browser)
 {
 	if(INFC_IS_BROWSER(browser))
 	{
-		std::auto_ptr<RequestInfo> info(new RequestInfo(
+		std::unique_ptr<RequestInfo> info(new RequestInfo(
 			*this, INF_BROWSER(browser), NULL, m_status_bar));
 
 		InfRequest* request = INF_REQUEST(
@@ -515,6 +544,11 @@ bool Gobby::BrowserCommands::create_chat_document(InfBrowser* browser)
 		iter->second->set_pending_chat(proxy);
 		return true;
 	}
+
+	// This cannot actually happen, but return to silence compiler
+	// warnings
+	g_assert_not_reached();
+	return false;
 }
 
 void Gobby::BrowserCommands::on_connect(const Glib::ustring& hostname)
@@ -553,7 +587,7 @@ void Gobby::BrowserCommands::on_activate(InfBrowser* browser,
 		// If there is already a request don't re-request
 		if(request == NULL)
 		{
-			std::auto_ptr<RequestInfo> info(new RequestInfo(
+			std::unique_ptr<RequestInfo> info(new RequestInfo(
 				*this, browser, iter, m_status_bar));
 
 			request = INF_REQUEST(
