@@ -1,5 +1,5 @@
 /* Gobby - GTK-based collaborative text editor
- * Copyright (C) 2008-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2008-2015 Armin Burgmeier <armin@arbur.net>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -16,14 +16,13 @@
 
 #include "commands/browser-context-commands.hpp"
 #include "operations/operation-open-multiple.hpp"
+#include "dialogs/connection-info-dialog.hpp"
 #include "util/file.hpp"
 #include "util/i18n.hpp"
 
-#include <gtkmm/icontheme.h>
-#include <gtkmm/imagemenuitem.h>
-#include <gtkmm/separatormenuitem.h>
-#include <gtkmm/stock.h>
+#include <gtkmm/builder.h>
 #include <giomm/file.h>
+#include <giomm/menu.h>
 #include <glibmm/fileutils.h>
 
 #include <libinfgtk/inf-gtk-permissions-dialog.h>
@@ -96,13 +95,54 @@ Gobby::BrowserContextCommands::BrowserContextCommands(
 	m_parent(parent), m_io(io), m_browser(browser),
 	m_file_chooser(chooser), m_operations(operations),
 	m_cert_manager(cert_manager), m_preferences(prefs),
-	m_popup_menu(NULL)
+	m_popup_menu(NULL),
+
+	m_action_group(Gio::SimpleActionGroup::create()),
+	m_action_remove(m_action_group->add_action("remove")),
+	m_action_connection_info(m_action_group->add_action("connection-info")),
+	m_action_disconnect(m_action_group->add_action("disconnect")),
+	m_action_create_account(m_action_group->add_action("create-account")),
+	m_action_create_document(
+		m_action_group->add_action("create-document")),
+	m_action_create_directory(
+		m_action_group->add_action("create-directory")),
+	m_action_open_document(m_action_group->add_action("open-document")),
+	m_action_permissions(m_action_group->add_action("permissions")),
+	m_action_delete(m_action_group->add_action("delete"))
 {
 	g_object_ref(io);
 
 	m_populate_popup_handler = g_signal_connect(
 		m_browser.get_view(), "populate-popup",
 		G_CALLBACK(on_populate_popup_static), this);
+
+	gtk_widget_insert_action_group(
+		GTK_WIDGET(m_browser.get_view()),
+		"browser", G_ACTION_GROUP(m_action_group->gobj()));
+
+	m_action_remove->signal_activate().connect(
+		sigc::mem_fun(*this, &BrowserContextCommands::on_remove));
+	m_action_connection_info->signal_activate().connect(
+		sigc::mem_fun(*this,
+			&BrowserContextCommands::on_connection_info));
+	m_action_disconnect->signal_activate().connect(
+		sigc::mem_fun(*this, &BrowserContextCommands::on_disconnect));
+	m_action_create_account->signal_activate().connect(
+		sigc::mem_fun(*this,
+			&BrowserContextCommands::on_create_account));
+	m_action_create_document->signal_activate().connect(
+		sigc::bind(sigc::mem_fun(*this,
+			&BrowserContextCommands::on_new), false));
+	m_action_create_directory->signal_activate().connect(
+		sigc::bind(sigc::mem_fun(*this,
+			&BrowserContextCommands::on_new), true));
+	m_action_open_document->signal_activate().connect(
+		sigc::mem_fun(*this, &BrowserContextCommands::on_open));
+	m_action_permissions->signal_activate().connect(
+		sigc::mem_fun(*this,
+			&BrowserContextCommands::on_permissions));
+	m_action_delete->signal_activate().connect(
+		sigc::mem_fun(*this, &BrowserContextCommands::on_delete));
 }
 
 Gobby::BrowserContextCommands::~BrowserContextCommands()
@@ -113,225 +153,92 @@ Gobby::BrowserContextCommands::~BrowserContextCommands()
 	g_object_unref(m_io);
 }
 
-void Gobby::BrowserContextCommands::on_menu_node_removed()
-{
-	g_assert(m_popup_menu != NULL);
-
-	// This calls deactivate, causing the watch to be removed.
-	m_popup_menu->popdown();
-}
-
-void Gobby::BrowserContextCommands::on_menu_deactivate()
-{
-	m_watch.reset(NULL);
-}
-
 void Gobby::BrowserContextCommands::on_populate_popup(Gtk::Menu* menu)
 {
 	// TODO: Can this happen? Should we close the old popup here?
 	g_assert(m_popup_menu == NULL);
+	g_assert(m_popup_watch.get() == NULL);
 
-	// Cancel previous attempts
-	m_watch.reset(NULL);
+	// Cancel previous dialogs
 	m_dialog.reset(NULL);
+	m_dialog_watch.reset(NULL);
 
 	InfBrowser* browser;
 	InfBrowserIter iter;
+	InfBrowserStatus status;
 
 	if(!m_browser.get_selected_browser(&browser))
 		return;
 
-	if(!m_browser.get_selected_iter(browser, &iter))
+	g_object_get(G_OBJECT(browser), "status", &status, NULL);
+	const bool has_iter = m_browser.get_selected_iter(browser, &iter);
+
+	// TODO: At the moment, action handlers access the browser item using
+	// the m_popup_watch variable. We should change it such that
+	// the selected item is passed as an action parameter, as two
+	// strings: The browser store item name, and the path to the selected
+	// node. This could then also easily allow browser actions to be
+	// invoked via d-bus.
+
+	Glib::RefPtr<Gtk::Builder> builder =
+		Gtk::Builder::create_from_resource(
+			"/de/0x539/gobby/ui/browser-context-menu.ui");
+
+	if(!has_iter)
 	{
 		InfBrowserStatus browser_status;
 		g_object_get(G_OBJECT(browser), "status",
 		             &browser_status, NULL);
 
+		Glib::RefPtr<Gio::Menu> menu_model =
+			Glib::RefPtr<Gio::Menu>::cast_dynamic(
+				builder->get_object(
+					"browser-context-menu-inactive"));
+
 		if(browser_status == INF_BROWSER_CLOSED)
 		{
-			Gtk::ImageMenuItem *remove_item = Gtk::manage(
-				new Gtk::ImageMenuItem(_("_Remove"), true));
-			remove_item->set_image(*Gtk::manage(new Gtk::Image(
-				Gtk::Stock::CLOSE, Gtk::ICON_SIZE_MENU)));
-			remove_item->signal_activate().connect(sigc::bind(
-				sigc::mem_fun(*this,
-					&BrowserContextCommands::on_remove),
-				browser));
-			remove_item->show();
-			menu->append(*remove_item);
+			menu->bind_model(menu_model, true);
 		}
-
-		return;
-	}
-
-	InfBrowserIter dummy_iter = iter;
-	bool is_subdirectory = inf_browser_is_subdirectory(browser, &iter);
-	bool is_toplevel = !inf_browser_get_parent(browser, &dummy_iter);
-
-	// Watch the node, and close the popup menu when the node
-	// it refers to is removed.
-	m_watch.reset(new NodeWatch(browser, &iter));
-	m_watch->signal_node_removed().connect(sigc::mem_fun(
-		*this, &BrowserContextCommands::on_menu_node_removed));
-
-	menu->signal_deactivate().connect(sigc::mem_fun(
-		*this, &BrowserContextCommands::on_menu_deactivate));
-
-	bool have_toplevel_entries = false;
-
-	// Add "Disconnect" menu option if the connection
-	// item has been clicked at
-	if(is_toplevel && INFC_IS_BROWSER(browser))
-	{
-		Gtk::ImageMenuItem* disconnect_item = Gtk::manage(
-			new Gtk::ImageMenuItem(
-				_("_Disconnect from Server"), true));
-		disconnect_item->set_image(*Gtk::manage(new Gtk::Image(
-			Gtk::Stock::DISCONNECT, Gtk::ICON_SIZE_MENU)));
-		disconnect_item->signal_activate().connect(sigc::bind(
-			sigc::mem_fun(*this,
-				&BrowserContextCommands::on_disconnect),
-			INFC_BROWSER(browser)));
-		disconnect_item->show();
-		menu->append(*disconnect_item);
-		have_toplevel_entries = true;
-	}
-
-	// Add create account option if the connection item has been clicked at
-	// Only offer this for clients at the moment, since local servers do
-	// not support this.
-	if(is_toplevel && INFC_IS_BROWSER(browser))
-	{
-		const InfAclAccount* account =
-			inf_browser_get_acl_local_account(browser);
-		const InfAclAccountId acc_id =
-			(account != NULL) ? account->id : 0;
-
-		InfAclMask mask;
-		inf_acl_mask_set1(&mask, INF_ACL_CAN_CREATE_ACCOUNT);
-		inf_browser_check_acl(browser, &iter, acc_id, &mask, &mask);
-		if(inf_acl_mask_has(&mask, INF_ACL_CAN_CREATE_ACCOUNT))
+		else
 		{
-			Gtk::Image* image = Gtk::manage(new Gtk::Image);
-			image->set_from_icon_name("application-certificate",
-			                          Gtk::ICON_SIZE_MENU);
-			Gtk::ImageMenuItem* item =
-				Gtk::manage(new Gtk::ImageMenuItem(
-	                                _("Create _Account..."), true));
-			item->set_image(*image);
-			item->signal_activate().connect(sigc::bind(
-				sigc::mem_fun(
-					*this,
-					&BrowserContextCommands::
-						on_create_account),
-				browser));
-			item->show();
-			menu->append(*item);
-			have_toplevel_entries = true;
+			// No popup menu
+			return;
 		}
 	}
-
-	// Separator, if we have entries dedicated to the browser itself
-	if(have_toplevel_entries)
-	{
-		Gtk::SeparatorMenuItem* sep_item =
-			Gtk::manage(new Gtk::SeparatorMenuItem);
-		sep_item->show();
-		menu->append(*sep_item);
-	}
-
-	// Create Document
-	Gtk::ImageMenuItem* new_document_item = Gtk::manage(
-		new Gtk::ImageMenuItem(_("Create Do_cument..."),
-		                       true));
-	new_document_item->set_image(*Gtk::manage(new Gtk::Image(
-		Gtk::Stock::NEW, Gtk::ICON_SIZE_MENU)));
-	new_document_item->signal_activate().connect(sigc::bind(
-		sigc::mem_fun(*this,
-			&BrowserContextCommands::on_new),
-		browser, iter, false));
-	new_document_item->set_sensitive(is_subdirectory);
-	new_document_item->show();
-	menu->append(*new_document_item);
-
-	// Create Directory
-
-	// Check whether we have the folder-new icon, fall back to
-	// Stock::DIRECTORY otherwise
-	Glib::RefPtr<Gdk::Screen> screen = menu->get_screen();
-	Glib::RefPtr<Gtk::IconTheme> icon_theme(
-		Gtk::IconTheme::get_for_screen(screen));
-
-	Gtk::Image* new_directory_image = Gtk::manage(new Gtk::Image);
-	if(icon_theme->lookup_icon("folder-new",
-	                           Gtk::ICON_SIZE_MENU,
-	                           Gtk::ICON_LOOKUP_USE_BUILTIN))
-	{
-		new_directory_image->set_from_icon_name(
-			"folder-new", Gtk::ICON_SIZE_MENU);
-	}
 	else
 	{
-		new_directory_image->set(
-			Gtk::Stock::DIRECTORY, Gtk::ICON_SIZE_MENU);
+		InfBrowserIter dummy_iter = iter;
+		const bool is_subdirectory =
+			inf_browser_is_subdirectory(browser, &iter);
+		const bool is_toplevel =
+			!inf_browser_get_parent(browser, &dummy_iter);
+
+		m_action_connection_info->set_enabled(
+			is_toplevel && status == INF_BROWSER_OPEN);
+		m_action_disconnect->set_enabled(
+			is_toplevel && INFC_IS_BROWSER(browser));
+		m_action_create_account->set_enabled(
+			is_toplevel && INFC_IS_BROWSER(browser));
+
+		m_action_create_document->set_enabled(is_subdirectory);
+		m_action_create_directory->set_enabled(is_subdirectory);
+		m_action_open_document->set_enabled(is_subdirectory);
+		m_action_delete->set_enabled(!is_toplevel);
+
+		Glib::RefPtr<Gio::Menu> menu_model =
+			Glib::RefPtr<Gio::Menu>::cast_dynamic(
+				builder->get_object(
+					"browser-context-menu-active"));
+
+		menu->bind_model(menu_model, true);
 	}
 
-	Gtk::ImageMenuItem* new_directory_item = Gtk::manage(
-		new Gtk::ImageMenuItem(_("Create Di_rectory..."),
-		                       true));
-	new_directory_item->set_image(*new_directory_image);
-	new_directory_item->signal_activate().connect(sigc::bind(
-		sigc::mem_fun(*this,
-			&BrowserContextCommands::on_new),
-		browser, iter, true));
-	new_directory_item->set_sensitive(is_subdirectory);
-	new_directory_item->show();
-	menu->append(*new_directory_item);
+	// Watch the node, and close the popup menu when the node
+	// it refers to is removed.
+	m_popup_watch.reset(new NodeWatch(browser, has_iter ? &iter : NULL));
+	m_popup_watch->signal_node_removed().connect(sigc::mem_fun(
+		*this, &BrowserContextCommands::on_menu_node_removed));
 
-	// Open Document
-	Gtk::ImageMenuItem* open_document_item = Gtk::manage(
-		new Gtk::ImageMenuItem(_("_Open Document..."), true));
-	open_document_item->set_image(*Gtk::manage(new Gtk::Image(
-		Gtk::Stock::OPEN, Gtk::ICON_SIZE_MENU)));
-	open_document_item->signal_activate().connect(sigc::bind(
-		sigc::mem_fun(*this,
-			&BrowserContextCommands::on_open),
-		browser, iter));
-	open_document_item->set_sensitive(is_subdirectory);
-	open_document_item->show();
-	menu->append(*open_document_item);
-
-	// Separator
-	Gtk::SeparatorMenuItem* sep_item =
-		Gtk::manage(new Gtk::SeparatorMenuItem);
-	sep_item->show();
-	menu->append(*sep_item);
-
-	// Permissions
-	Gtk::ImageMenuItem* permissions_item = Gtk::manage(
-		new Gtk::ImageMenuItem(_("_Permissions..."), true));
-	permissions_item->set_image(*Gtk::manage(new Gtk::Image(
-		Gtk::Stock::PROPERTIES, Gtk::ICON_SIZE_MENU)));
-	permissions_item->signal_activate().connect(sigc::bind(
-		sigc::mem_fun(*this,
-			&BrowserContextCommands::on_permissions),
-		browser, iter));
-	permissions_item->show();
-	menu->append(*permissions_item);
-
-	// Delete
-	Gtk::ImageMenuItem* delete_item = Gtk::manage(
-		new Gtk::ImageMenuItem(_("D_elete"), true));
-	delete_item->set_image(*Gtk::manage(new Gtk::Image(
-		Gtk::Stock::DELETE, Gtk::ICON_SIZE_MENU)));
-	delete_item->signal_activate().connect(sigc::bind(
-		sigc::mem_fun(*this,
-			&BrowserContextCommands::on_delete),
-		browser, iter));
-	delete_item->set_sensitive(!is_toplevel);
-	delete_item->show();
-	menu->append(*delete_item);
-	
 	m_popup_menu = menu;
 	menu->signal_selection_done().connect(
 		sigc::mem_fun(
@@ -339,13 +246,56 @@ void Gobby::BrowserContextCommands::on_populate_popup(Gtk::Menu* menu)
 			&BrowserContextCommands::on_popdown));
 }
 
+void Gobby::BrowserContextCommands::on_menu_node_removed()
+{
+	g_assert(m_popup_menu != NULL);
+
+	// This calls selection_done, causing the watch to be removed
+	// and the m_popup_menu pointer to be reset.
+	m_popup_menu->popdown();
+}
+
 void Gobby::BrowserContextCommands::on_popdown()
 {
 	m_popup_menu = NULL;
+	m_popup_watch.reset(NULL);
 }
 
-void Gobby::BrowserContextCommands::on_disconnect(InfcBrowser* browser)
+// Action handlers
+
+void Gobby::BrowserContextCommands::on_remove(const Glib::VariantBase& param)
 {
+	InfBrowser* browser = m_popup_watch->get_browser();
+
+	InfBrowserStatus status;
+	g_object_get(G_OBJECT(browser), "status", &status, NULL);
+
+	g_assert(status == INF_BROWSER_CLOSED);
+
+	// Reset the watch explicitly here in case we drop the last reference
+	// to the browser.
+	m_popup_watch.reset(NULL);
+	m_browser.remove_browser(browser);
+}
+
+void Gobby::BrowserContextCommands::on_connection_info(
+	const Glib::VariantBase& param)
+{
+	InfBrowser* browser = m_popup_watch->get_browser();
+
+	m_dialog = ConnectionInfoDialog::create(m_parent, browser);
+	m_dialog->add_button(_("_Close"), Gtk::RESPONSE_CLOSE);
+	m_dialog->signal_response().connect(
+		sigc::mem_fun(*this,
+			&BrowserContextCommands::
+				on_connection_info_response));
+	m_dialog->present();
+}
+
+void Gobby::BrowserContextCommands::on_disconnect(
+	const Glib::VariantBase& param)
+{
+	InfcBrowser* browser = INFC_BROWSER(m_popup_watch->get_browser());
 	InfXmlConnection* connection = infc_browser_get_connection(browser);
 	InfXmlConnectionStatus status;
 	g_object_get(G_OBJECT(connection), "status", &status, NULL);
@@ -357,17 +307,19 @@ void Gobby::BrowserContextCommands::on_disconnect(InfcBrowser* browser)
 	}
 }
 
-void Gobby::BrowserContextCommands::on_create_account(InfBrowser* browser)
+void Gobby::BrowserContextCommands::on_create_account(
+	const Glib::VariantBase& param)
 {
+	InfBrowser* browser = m_popup_watch->get_browser();
+
 	InfGtkAccountCreationDialog* dlg =
 		inf_gtk_account_creation_dialog_new(
 			m_parent.gobj(), static_cast<GtkDialogFlags>(0),
 			m_io, browser);
-	std::auto_ptr<Gtk::Dialog> account_creation_dialog(
+	std::unique_ptr<Gtk::Dialog> account_creation_dialog(
 		Glib::wrap(GTK_DIALOG(dlg)));
 
-	account_creation_dialog->add_button(
-		Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
+	account_creation_dialog->add_button(_("_Close"), Gtk::RESPONSE_CLOSE);
 
 	account_creation_dialog->signal_response().connect(
 		sigc::mem_fun(*this,
@@ -377,40 +329,30 @@ void Gobby::BrowserContextCommands::on_create_account(InfBrowser* browser)
 		G_OBJECT(account_creation_dialog->gobj()), "account-created",
 		G_CALLBACK(on_account_created_static), this);
 
-	m_dialog = account_creation_dialog;
+	m_dialog = std::move(account_creation_dialog);
 	m_dialog->present();
 }
 
-void Gobby::BrowserContextCommands::on_remove(InfBrowser* browser)
-{
-	InfBrowserStatus status;
-	g_object_get(G_OBJECT(browser), "status", &status, NULL);
-
-	g_assert(status == INF_BROWSER_CLOSED);
-
-	m_browser.remove_browser(browser);
-}
-
-void Gobby::BrowserContextCommands::on_new(InfBrowser* browser,
-                                           InfBrowserIter iter,
+void Gobby::BrowserContextCommands::on_new(const Glib::VariantBase& param,
                                            bool directory)
 {
-	m_watch.reset(new NodeWatch(browser, &iter));
-	m_watch->signal_node_removed().connect(sigc::mem_fun(
+	InfBrowser* browser = m_popup_watch->get_browser();
+	const InfBrowserIter iter = *m_popup_watch->get_browser_iter();
+
+	m_dialog_watch.reset(new NodeWatch(browser, &iter));
+	m_dialog_watch->signal_node_removed().connect(sigc::mem_fun(
 		*this, &BrowserContextCommands::on_dialog_node_removed));
 
-	std::auto_ptr<EntryDialog> entry_dialog(
-		new EntryDialog(
+	std::unique_ptr<EntryDialog> entry_dialog(
+		EntryDialog::create(
 			m_parent,
 			directory ? _("Choose a name for the directory")
 			          : _("Choose a name for the document"),
 			directory ? _("_Directory Name:")
 			          : _("_Document Name:")));
 
-	entry_dialog->add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-	entry_dialog->add_button(_("C_reate"), Gtk::RESPONSE_ACCEPT)
-		->set_image(*Gtk::manage(new Gtk::Image(
-			Gtk::Stock::NEW, Gtk::ICON_SIZE_BUTTON)));
+	entry_dialog->add_button(_("_Cancel"), Gtk::RESPONSE_CANCEL);
+	entry_dialog->add_button(_("C_reate"), Gtk::RESPONSE_ACCEPT);
 
 	entry_dialog->set_text(directory ? _("New Directory")
 	                                   : _("New Document"));
@@ -419,18 +361,20 @@ void Gobby::BrowserContextCommands::on_new(InfBrowser* browser,
 			&BrowserContextCommands::on_new_response),
 		browser, iter, directory));
 
-	m_dialog = entry_dialog;
+	m_dialog = std::move(entry_dialog);
 	m_dialog->present();
 }
 
-void Gobby::BrowserContextCommands::on_open(InfBrowser* browser,
-                                            InfBrowserIter iter)
+void Gobby::BrowserContextCommands::on_open(const Glib::VariantBase& param)
 {
-	m_watch.reset(new NodeWatch(browser, &iter));
-	m_watch->signal_node_removed().connect(sigc::mem_fun(
+	InfBrowser* browser = m_popup_watch->get_browser();
+	const InfBrowserIter iter = *m_popup_watch->get_browser_iter();
+
+	m_dialog_watch.reset(new NodeWatch(browser, &iter));
+	m_dialog_watch->signal_node_removed().connect(sigc::mem_fun(
 		*this, &BrowserContextCommands::on_dialog_node_removed));
 
-	std::auto_ptr<FileChooser::Dialog> file_dialog(
+	std::unique_ptr<FileChooser::Dialog> file_dialog(
 		new FileChooser::Dialog(
 			m_file_chooser, m_parent,
 			_("Choose a text file to open"),
@@ -442,49 +386,65 @@ void Gobby::BrowserContextCommands::on_open(InfBrowser* browser,
 
 	file_dialog->set_select_multiple(true);
 
-	m_dialog = file_dialog;
+	m_dialog = std::move(file_dialog);
 	m_dialog->present();
 }
 
-void Gobby::BrowserContextCommands::on_permissions(InfBrowser* browser,
-                                                   InfBrowserIter iter)
+void Gobby::BrowserContextCommands::on_permissions(
+	const Glib::VariantBase& param)
 {
-	m_watch.reset(new NodeWatch(browser, &iter));
-	m_watch->signal_node_removed().connect(sigc::mem_fun(
+	InfBrowser* browser = m_popup_watch->get_browser();
+	const InfBrowserIter iter = *m_popup_watch->get_browser_iter();
+
+	m_dialog_watch.reset(new NodeWatch(browser, &iter));
+	m_dialog_watch->signal_node_removed().connect(sigc::mem_fun(
 		*this, &BrowserContextCommands::on_dialog_node_removed));
 
 	InfGtkPermissionsDialog* dlg = inf_gtk_permissions_dialog_new(
 		m_parent.gobj(), static_cast<GtkDialogFlags>(0),
 		browser, &iter);
-	std::auto_ptr<Gtk::Dialog> permissions_dialog(
+	std::unique_ptr<Gtk::Dialog> permissions_dialog(
 		Glib::wrap(GTK_DIALOG(dlg)));
 
-	permissions_dialog->add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
+	permissions_dialog->add_button(_("_Close"), Gtk::RESPONSE_CLOSE);
 
 	permissions_dialog->signal_response().connect(
 		sigc::mem_fun(*this,
 			&BrowserContextCommands::on_permissions_response));
 
-	m_dialog = permissions_dialog;
+	m_dialog = std::move(permissions_dialog);
 	m_dialog->present();
 }
 
-void Gobby::BrowserContextCommands::on_delete(InfBrowser* browser,
-                                              InfBrowserIter iter)
+void Gobby::BrowserContextCommands::on_delete(
+	const Glib::VariantBase& param)
 {
-	m_operations.delete_node(browser, &iter);
+	InfBrowser* browser = m_popup_watch->get_browser();
+	const InfBrowserIter* iter = m_popup_watch->get_browser_iter();
+
+	m_operations.delete_node(browser, iter);
 }
 
+// Dialogs
+
 void Gobby::BrowserContextCommands::on_dialog_node_removed()
 {
-	m_watch.reset(NULL);
 	m_dialog.reset(NULL);
+	m_dialog_watch.reset(NULL);
 }
 
-void Gobby::BrowserContextCommands::on_create_account_response(int response_id)
+void Gobby::BrowserContextCommands::on_create_account_response(
+	int response_id)
 {
-	m_watch.reset(NULL);
 	m_dialog.reset(NULL);
+	m_dialog_watch.reset(NULL);
+}
+
+void Gobby::BrowserContextCommands::on_connection_info_response(
+	int response_id)
+{
+	m_dialog.reset(NULL);
+	m_dialog_watch.reset(NULL);
 }
 
 void Gobby::BrowserContextCommands::on_account_created(
@@ -530,11 +490,12 @@ void Gobby::BrowserContextCommands::on_account_created(
 	const unsigned int n_certs =
 		inf_certificate_chain_get_n_certificates(certificate);
 
-	std::auto_ptr<Gtk::MessageDialog> dlg;
+	std::unique_ptr<Gtk::MessageDialog> dlg;
 
 	try
 	{
-		const std::string filename = make_certificate_filename(cn, host);
+		const std::string filename =
+			make_certificate_filename(cn, host);
 
 		GError* error = NULL;
 		inf_cert_util_write_certificate_with_key(
@@ -590,7 +551,7 @@ void Gobby::BrowserContextCommands::on_account_created(
 			  "account: %1"), ex.what()));
 	}
 
-	m_dialog = dlg;
+	m_dialog = std::move(dlg);
 	m_dialog->signal_response().connect(
 		sigc::mem_fun(
 			*this,
@@ -627,8 +588,8 @@ void Gobby::BrowserContextCommands::on_new_response(int response_id,
 		}
 	}
 
-	m_watch.reset(NULL);
 	m_dialog.reset(NULL);
+	m_dialog_watch.reset(NULL);
 }
 
 void Gobby::BrowserContextCommands::on_open_response(int response_id,
@@ -639,21 +600,19 @@ void Gobby::BrowserContextCommands::on_open_response(int response_id,
 		static_cast<FileChooser::Dialog*>(m_dialog.get());
 	if(response_id == Gtk::RESPONSE_ACCEPT)
 	{
-		Glib::SListHandle<Glib::ustring> uris = dialog->get_uris();
-
-		OperationOpenMultiple::uri_list uri_list(
-			uris.begin(), uris.end());
+		const std::vector<Glib::RefPtr<Gio::File> > files =
+			dialog->get_files();
 
 		m_operations.create_documents(
-			browser, &iter, m_preferences, uri_list);
+			browser, &iter, m_preferences, files);
 	}
 
-	m_watch.reset(NULL);
 	m_dialog.reset(NULL);
+	m_dialog_watch.reset(NULL);
 }
 
 void Gobby::BrowserContextCommands::on_permissions_response(int response_id)
 {
-	m_watch.reset(NULL);
 	m_dialog.reset(NULL);
+	m_dialog_watch.reset(NULL);
 }
