@@ -1,5 +1,5 @@
 /* Gobby - GTK-based collaborative text editor
- * Copyright (C) 2008-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2008-2015 Armin Burgmeier <armin@arbur.net>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,10 +14,9 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "core/gobject/gobby-undo-manager.h"
 #include "core/textsessionview.hpp"
 #include "util/i18n.hpp"
-#include "util/color.hpp"
-#include "util/gtk-compat.hpp"
 
 #include <glibmm/main.h>
 #include <glibmm/markup.h>
@@ -25,11 +24,7 @@
 #include <gtkmm/scrolledwindow.h>
 #include <gtkmm/textiter.h>
 
-#include <gtksourceview/gtksourcebuffer.h>
-#include <gtksourceview/gtksourcelanguage.h>
-#include <gtksourceview/gtksourcelanguagemanager.h>
-#include <gtksourceview/gtksourcebuffer.h>
-
+#include <gtksourceview/gtksource.h>
 
 #include <libinftextgtk/inf-text-gtk-buffer.h>
 
@@ -145,19 +140,11 @@ Gobby::TextSessionView::TextSessionView(InfTextSession* session,
 		GTK_TEXT_VIEW(m_view),
 		user_table);
 
-#ifdef USE_GTKMM3
 	g_signal_connect_after(
 		G_OBJECT(m_view),
 		"style-updated",
-		G_CALLBACK(on_style_updated_static),
+		G_CALLBACK(on_view_style_updated_static),
 		this);
-#else
-	g_signal_connect_after(
-		G_OBJECT(m_view),
-		"style-set",
-		G_CALLBACK(on_style_set_static),
-		this);
-#endif
 
 	// This is a hack to make sure that the author tags in the textview
 	// have lowest priority of all tags, especially lower than
@@ -257,6 +244,8 @@ Gobby::TextSessionView::TextSessionView(InfTextSession* session,
 		m_infview,
 		m_preferences.user.show_remote_current_lines
 	);
+	inf_text_gtk_buffer_set_fade(
+		INF_TEXT_GTK_BUFFER(buffer), m_preferences.user.alpha);
 
 	gtk_source_view_set_tab_width(m_view, m_preferences.editor.tab_width);
 	gtk_source_view_set_insert_spaces_instead_of_tabs(
@@ -282,15 +271,13 @@ Gobby::TextSessionView::TextSessionView(InfTextSession* session,
 		m_buffer, m_preferences.view.bracket_highlight);
 	gtk_source_view_set_draw_spaces(
 		m_view, m_preferences.view.whitespace_display);
-	const Pango::FontDescription& desc = m_preferences.appearance.font;
-	gtk_widget_modify_font(
-		GTK_WIDGET(m_view),
-		const_cast<PangoFontDescription*>(desc.gobj()));
 
 	gtk_widget_show(GTK_WIDGET(m_view));
 	Gtk::ScrolledWindow* scroll = Gtk::manage(new Gtk::ScrolledWindow);
 	scroll->set_shadow_type(Gtk::SHADOW_IN);
 	scroll->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
+	scroll->set_hexpand(true);
+	scroll->set_vexpand(true);
 	gtk_container_add(GTK_CONTAINER(scroll->gobj()), GTK_WIDGET(m_view));
 	scroll->show();
 
@@ -300,7 +287,10 @@ Gobby::TextSessionView::TextSessionView(InfTextSession* session,
 		m_preferences.user.show_remote_cursor_positions
 	);
 
-	pack_start(*scroll, Gtk::PACK_EXPAND_WIDGET);
+	attach_next_to(*scroll, m_info_frame, Gtk::POS_BOTTOM, 1, 1);
+
+	// Set initial font
+	on_font_changed();
 }
 
 Gobby::TextSessionView::~TextSessionView()
@@ -412,10 +402,20 @@ void Gobby::TextSessionView::set_active_user(InfTextUser* user)
 				inf_adopted_session_get_algorithm(
 					INF_ADOPTED_SESSION(m_session)),
 				user, GTK_TEXT_BUFFER(m_buffer)));
+
+		GobbyUndoManager* undo_manager = gobby_undo_manager_new(
+			INF_TEXT_SESSION(m_session),
+			m_undo_grouping->get_inf_grouping());
+
+		gtk_source_buffer_set_undo_manager(
+			m_buffer,
+			GTK_SOURCE_UNDO_MANAGER(undo_manager));
+		g_object_unref(undo_manager);
 	}
 	else
 	{
 		m_undo_grouping.reset(NULL);
+		gtk_source_buffer_set_undo_manager(m_buffer, NULL);
 	}
 }
 
@@ -443,34 +443,11 @@ void Gobby::TextSessionView::on_user_color_changed()
 
 void Gobby::TextSessionView::on_alpha_changed()
 {
-#ifdef USE_GTKMM3
-	GtkStyleContext* style =
-		gtk_widget_get_style_context(GTK_WIDGET(m_view));
-	g_assert(style != NULL);
-
-	// Get the view background, not the one for the side panels:
-	gtk_style_context_save(style);
-	gtk_style_context_add_class(style, GTK_STYLE_CLASS_VIEW);
-
-	GdkRGBA rgba;
-	gtk_style_context_get_background_color(style, GTK_STATE_FLAG_NORMAL, &rgba);
-	gtk_style_context_restore(style);
-
-	Gdk::RGBA rgbacpp(&rgba);
-	const Gdk::Color colorcpp = GtkCompat::color_from_rgba(rgbacpp);
-	const GdkColor& color = *colorcpp.gobj();
-#else
-	GtkStyle* style = gtk_widget_get_style(GTK_WIDGET(m_view));
-	g_assert(style != NULL);
-
-	const GdkColor& color = style->base[GTK_STATE_NORMAL];
-#endif
-
 	InfTextGtkBuffer* buffer = INF_TEXT_GTK_BUFFER(
 		inf_session_get_buffer(INF_SESSION(m_session)));
 
 	inf_text_gtk_buffer_set_fade(
-		buffer, m_preferences.user.alpha, &color);
+		buffer, m_preferences.user.alpha);
 }
 
 void Gobby::TextSessionView::on_show_remote_cursors_changed()
@@ -576,9 +553,33 @@ void Gobby::TextSessionView::on_whitespace_display_changed()
 void Gobby::TextSessionView::on_font_changed()
 {
 	const Pango::FontDescription& desc = m_preferences.appearance.font;
-	gtk_widget_modify_font(
-		GTK_WIDGET(m_view),
-		const_cast<PangoFontDescription*>(desc.gobj()));
+
+	if(!m_font_provider)
+	{
+		m_font_provider = Gtk::CssProvider::create();
+		get_style_context()->add_provider(
+			m_font_provider,
+			GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	}
+
+	const Glib::ustring font_style = Glib::ustring::compose(
+		"* {\n"
+		"  font-size: %1pt;\n" 
+		"  font-family: \"%2\";\n"
+		"  font-weight: %3;\n"
+		"}",
+		desc.get_size() / PANGO_SCALE,
+		desc.get_family(),
+		desc.get_weight());
+
+	try
+	{
+		m_font_provider->load_from_data(font_style);
+	}
+	catch (const Glib::Error& ex)
+	{
+		g_warning("Failed to apply font settings: %s", ex.what().c_str());
+	}
 }
 
 void Gobby::TextSessionView::on_scheme_changed()
@@ -647,9 +648,8 @@ bool Gobby::TextSessionView::
 	return true;
 }
 
-void Gobby::TextSessionView::on_style_set()
+void Gobby::TextSessionView::on_view_style_updated()
 {
-#ifdef USE_GTKMM3
 	GtkStyleContext* style =
 		gtk_widget_get_style_context(GTK_WIDGET(m_view));
 	g_assert(style != NULL);
@@ -659,32 +659,20 @@ void Gobby::TextSessionView::on_style_set()
 	gtk_style_context_add_class(style, GTK_STYLE_CLASS_VIEW);
 
 	GdkRGBA rgba;
-	gtk_style_context_get_background_color(style, GTK_STATE_FLAG_NORMAL, &rgba);
+	gtk_style_context_get_background_color(
+		style, GTK_STATE_FLAG_NORMAL, &rgba);
 	gtk_style_context_restore(style);
 
-	Gdk::RGBA rgbacpp(&rgba);
-	const Gdk::Color colorcpp = GtkCompat::color_from_rgba(rgbacpp);
-	const GdkColor& color = *colorcpp.gobj();
-#else
-	GtkStyle* style = gtk_widget_get_style(GTK_WIDGET(m_view));
-	g_assert(style != NULL);
+	// Convert background color to HSV
+	double h, s, v;
+	gtk_rgb_to_hsv(rgba.red, rgba.green, rgba.blue, &h, &s, &v);
 
-	const GdkColor& color = style->base[GTK_STATE_NORMAL];
-#endif
-
-	double rh = color.red / 65535.0;
-	double gs = color.green / 65535.0;
-	double bv = color.blue / 65535.0;
-
-	Gobby::rgb_to_hsv(rh, gs, bv);
-
-	double my_sat = gs * 0.5 + 0.3;
-	double my_val = (std::pow(bv + 1, 3) - 1) / 7 * 0.6 + 0.4;
+	// Adjust S and V for authorship colors
+	s = s * 0.5 + 0.3;
+	v = (std::pow(v + 1, 3) - 1) / 7 * 0.6 + 0.4;
 
 	InfTextGtkBuffer* buffer = INF_TEXT_GTK_BUFFER(
 		inf_session_get_buffer(INF_SESSION(m_session)));
 
-	inf_text_gtk_buffer_set_saturation_value(buffer, my_sat, my_val);
-	inf_text_gtk_buffer_set_fade(
-		buffer, m_preferences.user.alpha, &color);
+	inf_text_gtk_buffer_set_saturation_value(buffer, s, v);
 }
