@@ -1,5 +1,5 @@
 /* Gobby - GTK-based collaborative text editor
- * Copyright (C) 2008-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2008-2015 Armin Burgmeier <armin@arbur.net>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -17,18 +17,16 @@
 #include "dialogs/find-dialog.hpp"
 #include "core/folder.hpp"
 #include "util/i18n.hpp"
-#include "util/gtk-compat.hpp"
 
 #include <gtkmm/messagedialog.h>
 #include <gtkmm/textbuffer.h>
-#include <gtkmm/stock.h>
 
 namespace
 {
 	typedef gboolean (*TextSearchFunc)(
 		const GtkTextIter*,
 		const gchar*,
-		Gobby::GtkCompat::TextSearchFlags,
+		GtkTextSearchFlags,
 		GtkTextIter*,
 		GtkTextIter*,
 		const GtkTextIter*
@@ -39,77 +37,56 @@ namespace
 	const int RESPONSE_REPLACE_ALL = 3;
 }
 
-Gobby::FindDialog::FindDialog(Gtk::Window& parent, const Folder& folder,
-                              StatusBar& status_bar):
-	Gtk::Dialog(_("Find"), parent),
-	m_folder(folder), m_status_bar(status_bar),
-	m_table_entries(2, 2),
-	m_label_find(_("_Search for:"), GtkCompat::ALIGN_LEFT,
-	             Gtk::ALIGN_CENTER, true),
-	m_label_replace(_("Replace _with:"), GtkCompat::ALIGN_LEFT,
-	                Gtk::ALIGN_CENTER, true),
-	m_check_case(_("_Match case"), true),
-	m_check_whole_word(_("Match _entire word only"), true),
-	m_check_backwards(_("Search _backwards"), true),
-	m_check_wrap_around(_("Wra_p around"), true)
+Gobby::FindDialog::FindDialog(GtkDialog* cobject,
+                              const Glib::RefPtr<Gtk::Builder>& builder):
+	Gtk::Dialog(cobject), m_folder(NULL), m_status_bar(NULL)
 {
-	m_entry_find.set_activates_default(true);
-	m_entry_replace.set_activates_default(true);
-	m_label_find.set_mnemonic_widget(m_entry_find);
-	m_label_replace.set_mnemonic_widget(m_entry_replace);
+	builder->get_widget("search-for", m_entry_find);
+	builder->get_widget("replace-with-label", m_label_replace);
+	builder->get_widget("replace-with", m_entry_replace);
 
-	m_label_find.show();
-	m_entry_find.show();
+	builder->get_widget("match-case", m_check_case);
+	builder->get_widget("match-entire-word-only", m_check_whole_word);
+	builder->get_widget("search-backwards", m_check_backwards);
+	builder->get_widget("wrap-around", m_check_wrap_around);
 
-	m_table_entries.attach(m_label_find, 0, 1, 0, 1,
-	                       Gtk::FILL, Gtk::FILL);
-	m_table_entries.attach(m_label_replace, 0, 1, 1, 2,
-	                       Gtk::FILL, Gtk::FILL);
-	m_table_entries.attach(m_entry_find, 1, 2, 0, 1,
-	                       Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK);
-	m_table_entries.attach(m_entry_replace, 1, 2, 1, 2,
-	                       Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK);
-	m_table_entries.show();
-	m_table_entries.set_spacings(12);
-
-	m_check_case.show();
-	m_check_whole_word.show();
-	m_check_backwards.show();
-	m_check_wrap_around.show();
-	m_check_wrap_around.set_active(true);
-
-	m_box_main.pack_start(m_table_entries, Gtk::PACK_SHRINK);
-	m_box_main.pack_start(m_check_case, Gtk::PACK_SHRINK);
-	m_box_main.pack_start(m_check_whole_word, Gtk::PACK_SHRINK);
-	m_box_main.pack_start(m_check_backwards, Gtk::PACK_SHRINK);
-	m_box_main.pack_start(m_check_wrap_around, Gtk::PACK_SHRINK);
-	m_box_main.set_spacing(12);
-	m_box_main.show();
-
-	get_vbox()->pack_start(m_box_main);
-
-	m_entry_find.signal_changed().connect(
+	m_entry_find->signal_changed().connect(
 		sigc::mem_fun(*this, &FindDialog::on_find_text_changed));
-	m_entry_replace.signal_changed().connect(
+	m_entry_replace->signal_changed().connect(
 		sigc::mem_fun(*this, &FindDialog::on_replace_text_changed));
 
-	set_border_width(12);
-	set_resizable(false);
-
-	add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
+	add_button(_("_Close"), Gtk::RESPONSE_CLOSE);
 	m_button_replace_all = add_button(_("Replace _All"),
 	                                  RESPONSE_REPLACE_ALL);
 	m_button_replace = add_button(_("_Replace"), RESPONSE_REPLACE);
-	add_button(Gtk::Stock::FIND, RESPONSE_FIND);
+	add_button(_("_Find"), RESPONSE_FIND);
 
 	set_default_response(RESPONSE_FIND);
+}
 
-	m_button_replace->set_image(*Gtk::manage(new Gtk::Image(
-		Gtk::Stock::FIND_AND_REPLACE, Gtk::ICON_SIZE_BUTTON)));
+std::unique_ptr<Gobby::FindDialog>
+Gobby::FindDialog::create(Gtk::Window& parent, const Folder& folder,
+                          StatusBar& status_bar)
+{
+	Glib::RefPtr<Gtk::Builder> builder =
+		Gtk::Builder::create_from_resource(
+			"/de/0x539/gobby/ui/find-dialog.ui");
+
+	FindDialog* dialog_ptr;
+	builder->get_widget_derived("FindDialog", dialog_ptr);
+	std::unique_ptr<FindDialog> dialog(dialog_ptr);
+
+	dialog->set_transient_for(parent);
+	dialog->m_folder = &folder;
+	dialog->m_status_bar = &status_bar;
+
+	folder.signal_document_changed().connect(
+		sigc::mem_fun(*dialog, &FindDialog::on_document_changed));
+
+	// For initial sensitivity:
+	dialog->on_document_changed(folder.get_current_document());
+	return dialog;
 
-	m_folder.signal_document_changed().connect(
-		sigc::mem_fun(*this, &FindDialog::on_document_changed));
-	on_document_changed(m_folder.get_current_document());
 }
 
 Gobby::FindDialog::~FindDialog()
@@ -119,22 +96,22 @@ Gobby::FindDialog::~FindDialog()
 
 bool Gobby::FindDialog::get_search_only() const
 {
-	return GtkCompat::is_visible(m_label_replace);
+	return m_label_replace->get_visible();
 }
 
 void Gobby::FindDialog::set_search_only(bool search_only)
 {
 	if(search_only)
 	{
-		m_label_replace.hide();
-		m_entry_replace.hide();
+		m_label_replace->hide();
+		m_entry_replace->hide();
 		m_button_replace->hide();
 		m_button_replace_all->hide();
 	}
 	else
 	{
-		m_label_replace.show();
-		m_entry_replace.show();
+		m_label_replace->show();
+		m_entry_replace->show();
 		m_button_replace->show();
 		m_button_replace_all->show();
 	}
@@ -144,12 +121,12 @@ void Gobby::FindDialog::set_search_only(bool search_only)
 
 Glib::ustring Gobby::FindDialog::get_find_text() const
 {
-	return m_entry_find.get_text();
+	return m_entry_find->get_text();
 }
 
 Glib::ustring Gobby::FindDialog::get_replace_text() const
 {
-	return m_entry_replace.get_text();
+	return m_entry_replace->get_text();
 }
 
 bool Gobby::FindDialog::find_next()
@@ -161,7 +138,7 @@ bool Gobby::FindDialog::find_next()
 			_("Phrase \"%1\" has not been found"),
 			get_find_text());
 
-		m_status_bar.add_info_message(str, 5);
+		m_status_bar->add_info_message(str, 5);
 		return false;
 	}
 
@@ -177,7 +154,7 @@ bool Gobby::FindDialog::find_previous()
 			_("Phrase \"%1\" has not been found"),
 			get_find_text());
 
-		m_status_bar.add_info_message(str, 5);
+		m_status_bar->add_info_message(str, 5);
 		return false;
 	}
 
@@ -187,7 +164,7 @@ bool Gobby::FindDialog::find_previous()
 void Gobby::FindDialog::on_show()
 {
 	Gtk::Dialog::on_show();
-	m_entry_find.grab_focus();
+	m_entry_find->grab_focus();
 }
 
 void Gobby::FindDialog::on_response(int id)
@@ -246,7 +223,7 @@ void Gobby::FindDialog::on_replace_text_changed()
 
 Gobby::FindDialog::SearchDirection Gobby::FindDialog::get_direction() const
 {
-	if(m_check_backwards.get_active())
+	if(m_check_backwards->get_active())
 		return SEARCH_BACKWARD;
 	else
 		return SEARCH_FORWARD;
@@ -262,7 +239,7 @@ bool Gobby::FindDialog::find()
 
 bool Gobby::FindDialog::replace()
 {
-	SessionView* view = m_folder.get_current_document();
+	SessionView* view = m_folder->get_current_document();
 	TextSessionView* text_view = dynamic_cast<TextSessionView*>(view);
 	g_assert(text_view != NULL);
 
@@ -271,7 +248,7 @@ bool Gobby::FindDialog::replace()
 	Glib::ustring find_str = get_find_text();
 
 	// Lowercase both if we are comparing insensitive
-	if(!m_check_case.get_active() )
+	if(!m_check_case->get_active() )
 	{
 		sel_str = sel_str.casefold();
 		find_str = find_str.casefold();
@@ -304,7 +281,7 @@ bool Gobby::FindDialog::replace_all()
 {
 	// TODO: Add helper function to get textsessionview? Maybe even add
 	// to Folder?
-	SessionView* view = m_folder.get_current_document();
+	SessionView* view = m_folder->get_current_document();
 	TextSessionView* text_view = dynamic_cast<TextSessionView*>(view);
 	g_assert(text_view != NULL);
 
@@ -344,14 +321,14 @@ bool Gobby::FindDialog::replace_all()
 		result = true;
 	}
 
-	m_status_bar.add_info_message(message, 5);
+	m_status_bar->add_info_message(message, 5);
 	return result;
 }
 
 bool Gobby::FindDialog::find_and_select(const GtkTextIter* from,
                                         SearchDirection direction)
 {
-	SessionView* view = m_folder.get_current_document();
+	SessionView* view = m_folder->get_current_document();
 	TextSessionView* text_view = dynamic_cast<TextSessionView*>(view);
 	g_assert(text_view != NULL);
 
@@ -387,7 +364,7 @@ bool Gobby::FindDialog::find_wrap(const GtkTextIter* from,
                                   GtkTextIter* match_start,
                                   GtkTextIter* match_end)
 {
-	SessionView* view = m_folder.get_current_document();
+	SessionView* view = m_folder->get_current_document();
 	TextSessionView* text_view = dynamic_cast<TextSessionView*>(view);
 	g_assert(text_view != NULL);
 
@@ -397,7 +374,7 @@ bool Gobby::FindDialog::find_wrap(const GtkTextIter* from,
 	                         match_start, match_end);
 	if(result == true) return true;
 
-	if(!m_check_wrap_around.get_active()) return false;
+	if(!m_check_wrap_around->get_active()) return false;
 
 	// Wrap around
 	GtkTextIter restart_pos;
@@ -433,7 +410,7 @@ bool Gobby::FindDialog::find_range(const GtkTextIter* from,
 	while(find_range_once(&start_pos, to, direction,
 	                      match_start, match_end))
 	{
-		if(m_check_whole_word.get_active() )
+		if(m_check_whole_word->get_active() )
 		{
 			if(!gtk_text_iter_starts_word(match_start) ||
 			   !gtk_text_iter_ends_word(match_end))
@@ -459,14 +436,14 @@ bool Gobby::FindDialog::find_range_once(const GtkTextIter* from,
                                         GtkTextIter* match_start,
                                         GtkTextIter* match_end)
 {
-	GtkCompat::TextSearchFlags flags = GtkCompat::TextSearchFlags(0);
-	if(!m_check_case.get_active() )
-		flags = GtkCompat::TEXT_SEARCH_CASE_INSENSITIVE;
+	GtkTextSearchFlags flags = GtkTextSearchFlags(0);
+	if(!m_check_case->get_active() )
+		flags = GTK_TEXT_SEARCH_CASE_INSENSITIVE;
 
 	TextSearchFunc search_func = (direction == SEARCH_FORWARD ?
-		GtkCompat::text_iter_forward_search :
-		GtkCompat::text_iter_backward_search);
-	Glib::ustring find_str = m_entry_find.get_text();
+		gtk_text_iter_forward_search :
+		gtk_text_iter_backward_search);
+	Glib::ustring find_str = m_entry_find->get_text();
 
 	gboolean result = search_func(
 		from,
@@ -482,11 +459,11 @@ bool Gobby::FindDialog::find_range_once(const GtkTextIter* from,
 
 void Gobby::FindDialog::update_sensitivity()
 {
-	SessionView* view = m_folder.get_current_document();
+	SessionView* view = m_folder->get_current_document();
 	TextSessionView* text_view = dynamic_cast<TextSessionView*>(view);
 
 	bool find_sensitivity =
-		(!m_entry_find.get_text().empty() && text_view != NULL);
+		(!m_entry_find->get_text().empty() && text_view != NULL);
 	bool replace_sensitivity =
 		(find_sensitivity && text_view->get_active_user() != NULL);
 
