@@ -18,8 +18,7 @@
 #define _GOBBY_OPENLOCATIONDIALOG_HPP_
 
 #include <gtkmm/dialog.h>
-#include <gtkmm/label.h>
-#include <gtkmm/box.h>
+#include <gtkmm/builder.h>
 
 #include "util/historyentry.hpp"
 
@@ -28,8 +27,13 @@ namespace Gobby
 
 class OpenLocationDialog: public Gtk::Dialog
 {
+private:
+	friend class Gtk::Builder;
+	OpenLocationDialog(GtkDialog* cobject,
+	                   const Glib::RefPtr<Gtk::Builder>& builder);
+
 public:
-	OpenLocationDialog(Gtk::Window& parent);
+	static std::unique_ptr<OpenLocationDialog> create(Gtk::Window& parent);
 
 	Glib::ustring get_uri() const;
 	void set_uri(const Glib::ustring& uri);
@@ -40,9 +44,7 @@ protected:
 
 	void on_entry_changed();
 
-	Gtk::VBox m_box;
-	Gtk::Label m_label;
-	HistoryComboBoxEntry m_combo;
+	HistoryComboBox m_combo;
 };
 
 }
